# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import print_function, unicode_literals

from .lang_EU import Num2Word_EU


class Num2Word_ES(Num2Word_EU):
    CURRENCY_FORMS = {
        'EUR': (('euro', 'euros'), ('céntimo', 'céntimos')),
        'ESP': (('peseta', 'pesetas'), ('céntimo', 'céntimos')),
        'USD': (('dolar', 'dólares'), ('centavo', 'centavos')),
        'PEN': (('sol', 'soles'), ('céntimo', 'céntimos')),
    }

    # //CHECK: Is this sufficient??
    GIGA_SUFFIX = None
    MEGA_SUFFIX = "illón"

    def setup(self):
        lows = ["cuatr", "tr", "b", "m"]
        self.high_numwords = self.gen_high_numwords([], [], lows)
        self.negword = "menos "
        self.pointword = "punto"
        self.errmsg_nonnum = "Solo números pueden ser convertidos a palabras."
        self.errmsg_toobig = (
            "Numero muy grande para ser convertido a palabras."
            )
        self.gender_stem = "o"
        self.exclude_title = ["y", "menos", "punto"]
        self.mid_numwords = [(1000, "mil"), (100, "cien"), (90, "noventa"),
                             (80, "ochenta"), (70, "setenta"), (60, "sesenta"),
                             (50, "cincuenta"), (40, "cuarenta"),
                             (30, "treinta")]
        self.low_numwords = ["veintinueve", "veintiocho", "veintisiete",
                             "veintiséis", "veinticinco", "veinticuatro",
                             "veintitrés", "veintidós", "veintiuno",
                             "veinte", "diecinueve", "dieciocho", "diecisiete",
                             "dieciseis", "quince", "catorce", "trece", "doce",
                             "once", "diez", "nueve", "ocho", "siete", "seis",
                             "cinco", "cuatro", "tres", "dos", "uno", "cero"]
        self.ords = {1: "primer",
                     2: "segund",
                     3: "tercer",
                     4: "cuart",
                     5: "quint",
                     6: "sext",
                     7: "séptim",
                     8: "octav",
                     9: "noven",
                     10: "décim",
                     20: "vigésim",
                     30: "trigésim",
                     40: "quadragésim",
                     50: "quincuagésim",
                     60: "sexagésim",
                     70: "septuagésim",
                     80: "octogésim",
                     90: "nonagésim",
                     100: "centésim",
                     200: "ducentésim",
                     300: "tricentésim",
                     400: "cuadrigentésim",
                     500: "quingentésim",
                     600: "sexcentésim",
                     700: "septigentésim",
                     800: "octigentésim",
                     900: "noningentésim",
                     1e3: "milésim",
                     1e6: "millonésim",
                     1e9: "billonésim",
                     1e12: "trillonésim",
                     1e15: "cuadrillonésim"}

    def merge(self, curr, next):
        ctext, cnum, ntext, nnum = curr + next

        if cnum == 1:
            if nnum < 1000000:
                return next
            ctext = "un"
        elif cnum == 100 and not nnum % 1000 == 0:
            ctext += "t" + self.gender_stem

        if nnum < cnum:
            if cnum < 100:
                return "%s y %s" % (ctext, ntext), cnum + nnum
            return "%s %s" % (ctext, ntext), cnum + nnum
        elif (not nnum % 1000000) and cnum > 1:
            ntext = ntext[:-3] + "lones"

        if nnum == 100:
            if cnum == 5:
                ctext = "quinien"
                ntext = ""
            elif cnum == 7:
                ctext = "sete"
            elif cnum == 9:
                ctext = "nove"
            ntext += "t" + self.gender_stem + "s"
        else:
            ntext = " " + ntext

        return (ctext + ntext, cnum * nnum)

    def to_ordinal(self, value):
        self.verify_ordinal(value)
        try:
            if value == 0:
                text = ""
            elif value <= 10:
                text = "%s%s" % (self.ords[value], self.gender_stem)
            elif value <= 12:
                text = (
                    "%s%s%s" % (self.ords[10], self.gender_stem,
                                self.to_ordinal(value - 10))
                        )
            elif value <= 100:
                dec = (value // 10) * 10
                text = (
                    "%s%s %s" % (self.ords[dec], self.gender_stem,
                                 self.to_ordinal(value - dec))
                        )
            elif value <= 1e3:
                cen = (value // 100) * 100
                text = (
                    "%s%s %s" % (self.ords[cen], self.gender_stem,
                                 self.to_ordinal(value - cen))
                        )
            elif value < 1e18:
                # dec contains the following:
                # [ 1e3,  1e6): 1e3
                # [ 1e6,  1e9): 1e6
                # [ 1e9, 1e12): 1e9
                # [1e12, 1e15): 1e12
                # [1e15, 1e18): 1e15
                dec = 10 ** ((((len(str(int(value))) - 1) / 3 - 1) + 1) * 3)
                part = int(float(value / dec) * dec)
                cardinal = (
                    self.to_cardinal(part / dec) if part / dec != 1 else ""
                    )
                text = (
                    "%s%s%s %s" % (cardinal, self.ords[dec], self.gender_stem,
                                   self.to_ordinal(value - part))
                        )
            else:
                text = self.to_cardinal(value)
        except KeyError:
            text = self.to_cardinal(value)
        return text.strip()

    def to_ordinal_num(self, value):
        self.verify_ordinal(value)
        return "%s%s" % (value, "º" if self.gender_stem == 'o' else "ª")

    def to_currency(self, val, currency='EUR', cents=True, separator=' con',
                    adjective=False):
        result = super(Num2Word_ES, self).to_currency(
            val, currency=currency, cents=cents, separator=separator,
            adjective=adjective)
        # Handle exception, in spanish is "un euro" and not "uno euro"
        return result.replace("uno", "un")



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words

TEST_CASES_CARDINAL = (
    (1, 'uno'),
    (2, 'dos'),
    (3, 'tres'),
    (5.5, 'cinco punto cinco'),
    (11, 'once'),
    (12, 'doce'),
    (16, 'dieciseis'),
    (17.42, 'diecisiete punto cuatro dos'),
    (19, 'diecinueve'),
    (20, 'veinte'),
    (21, 'veintiuno'),
    (26, 'veintiséis'),
    (27.312, 'veintisiete punto tres uno dos'),
    (28, 'veintiocho'),
    (30, 'treinta'),
    (31, 'treinta y uno'),
    (40, 'cuarenta'),
    (44, 'cuarenta y cuatro'),
    (50, 'cincuenta'),
    (53.486, 'cincuenta y tres punto cuatro ocho seis'),
    (55, 'cincuenta y cinco'),
    (60, 'sesenta'),
    (67, 'sesenta y siete'),
    (70, 'setenta'),
    (79, 'setenta y nueve'),
    (89, 'ochenta y nueve'),
    (95, 'noventa y cinco'),
    (100, 'cien'),
    (101, 'ciento uno'),
    (199, 'ciento noventa y nueve'),
    (203, 'doscientos tres'),
    (287, 'doscientos ochenta y siete'),
    (300.42, 'trescientos punto cuatro dos'),
    (356, 'trescientos cincuenta y seis'),
    (400, 'cuatrocientos'),
    (434, 'cuatrocientos treinta y cuatro'),
    (578, 'quinientos setenta y ocho'),
    (689, 'seiscientos ochenta y nueve'),
    (729, 'setecientos veintinueve'),
    (894, 'ochocientos noventa y cuatro'),
    (999, 'novecientos noventa y nueve'),
    (1000, 'mil'),
    (1001, 'mil uno'),
    (1097, 'mil noventa y siete'),
    (1104, 'mil ciento cuatro'),
    (1243, 'mil doscientos cuarenta y tres'),
    (2385, 'dos mil trescientos ochenta y cinco'),
    (3766, 'tres mil setecientos sesenta y seis'),
    (4196, 'cuatro mil ciento noventa y seis'),
    (4196.42, 'cuatro mil ciento noventa y seis punto cuatro dos'),
    (5846, 'cinco mil ochocientos cuarenta y seis'),
    (6459, 'seis mil cuatrocientos cincuenta y nueve'),
    (7232, 'siete mil doscientos treinta y dos'),
    (8569, 'ocho mil quinientos sesenta y nueve'),
    (9539, 'nueve mil quinientos treinta y nueve'),
    (1000000, 'un millón'),
    (1000001, 'un millón uno'),
    (4000000, 'cuatro millones'),
    (10000000000000, 'diez billones'),
    (100000000000000, 'cien billones'),
    (1000000000000000000, 'un trillón'),
    (1000000000000000000000, 'mil trillones'),
    (10000000000000000000000000, 'diez cuatrillones')
)

TEST_CASES_ORDINAL = (
    (1, 'primero'),
    (8, 'octavo'),
    (12, 'décimosegundo'),
    (14, 'décimo cuarto'),
    (28, 'vigésimo octavo'),
    (100, 'centésimo'),
    (1000, 'milésimo'),
    (1000000, 'millonésimo'),
    (1000000000000000, 'cuadrillonésimo'),
    (1000000000000000000, 'un trillón')  # over 1e18 is not supported
)

TEST_CASES_ORDINAL_NUM = (
    (1, '1º'),
    (8, '8º'),
    (12, '12º'),
    (14, '14º'),
    (28, '28º'),
    (100, '100º'),
    (1000, '1000º'),
    (1000000, '1000000º')
)

TEST_CASES_TO_CURRENCY = (
    (1.00, 'un euro con cero céntimos'),
    (2.00, 'dos euros con cero céntimos'),
    (8.00, 'ocho euros con cero céntimos'),
    (12.00, 'doce euros con cero céntimos'),
    (21.00, 'veintiun euros con cero céntimos'),
    (81.25, 'ochenta y un euros con veinticinco céntimos'),
    (350.90, 'trescientos cincuenta euros con noventa céntimos'),
    (100.00, 'cien euros con cero céntimos'),
)

TEST_CASES_TO_CURRENCY_ESP = (
    (1.00, 'un peseta con cero céntimos'),
    (2.00, 'dos pesetas con cero céntimos'),
    (8.00, 'ocho pesetas con cero céntimos'),
    (12.00, 'doce pesetas con cero céntimos'),
    (21.00, 'veintiun pesetas con cero céntimos'),
    (81.25, 'ochenta y un pesetas con veinticinco céntimos'),
    (350.90, 'trescientos cincuenta pesetas con noventa céntimos'),
    (100.00, 'cien pesetas con cero céntimos'),
)

TEST_CASES_TO_CURRENCY_USD = (
    (1.00, 'un dolar con cero centavos'),
    (2.00, 'dos dólares con cero centavos'),
    (8.00, 'ocho dólares con cero centavos'),
    (12.00, 'doce dólares con cero centavos'),
    (21.00, 'veintiun dólares con cero centavos'),
    (81.25, 'ochenta y un dólares con veinticinco centavos'),
    (350.90, 'trescientos cincuenta dólares con noventa centavos'),
    (100.00, 'cien dólares con cero centavos'),
)

TEST_CASES_TO_CURRENCY_PEN = (
    (1.00, 'un sol con cero céntimos'),
    (2.00, 'dos soles con cero céntimos'),
    (8.00, 'ocho soles con cero céntimos'),
    (12.00, 'doce soles con cero céntimos'),
    (21.00, 'veintiun soles con cero céntimos'),
    (81.25, 'ochenta y un soles con veinticinco céntimos'),
    (350.90, 'trescientos cincuenta soles con noventa céntimos'),
    (100.00, 'cien soles con cero céntimos'),
)


class Num2WordsESTest(TestCase):

    def test_number(self):
        for test in TEST_CASES_CARDINAL:
            self.assertEqual(num2words(test[0], lang='es'), test[1])

    def test_ordinal(self):
        for test in TEST_CASES_ORDINAL:
            self.assertEqual(
                num2words(test[0], lang='es', ordinal=True),
                test[1]
            )

    def test_ordinal_num(self):
        for test in TEST_CASES_ORDINAL_NUM:
            self.assertEqual(
                num2words(test[0], lang='es', to='ordinal_num'),
                test[1]
            )

    def test_currency(self):
        for test in TEST_CASES_TO_CURRENCY:
            self.assertEqual(
                num2words(test[0], lang='es', to='currency'),
                test[1]
            )

    def test_currency_esp(self):
        for test in TEST_CASES_TO_CURRENCY_ESP:
            self.assertEqual(
                num2words(test[0], lang='es', to='currency', currency='ESP'),
                test[1]
            )

    def test_currency_usd(self):
        for test in TEST_CASES_TO_CURRENCY_USD:
            self.assertEqual(
                num2words(test[0], lang='es', to='currency', currency='USD'),
                test[1]
            )

    def test_currency_pen(self):
        for test in TEST_CASES_TO_CURRENCY_PEN:
            self.assertEqual(
                num2words(test[0], lang='es', to='currency', currency='PEN'),
                test[1]
            )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from unittest import TestCase

from num2words import num2words


class Num2WordsUKTest(TestCase):
    def test_to_cardinal(self):
        self.maxDiff = None
        self.assertEqual(num2words(100, lang='uk'), 'сто')
        # self.assertEqual(num2words(101, lang='uk'), 'сто один')
        self.assertEqual(num2words(110, lang='uk'), 'сто десять')
        self.assertEqual(num2words(115, lang='uk'), "сто п'ятнадцять")
        self.assertEqual(num2words(123, lang='uk'), 'сто двадцять три')
        self.assertEqual(num2words(1000, lang='uk'), 'одна тисяча')
        # self.assertEqual(num2words(1001, lang='uk'), 'одна тисяча один')
        self.assertEqual(num2words(2012, lang='uk'), 'двi тисячi дванадцять')
        self.assertEqual(
            num2words(12519.85, lang='uk'),
            "дванадцять тисяч п'ятсот дев'ятнадцять кома вiсiмдесят п'ять")
        # self.assertEqual(
        #    num2words(1234567890, lang='uk'),
        #    "мiльярд двiстi тридцать чотири мiльйона п'ятсот шiстдесят сiмь "
        #    "тисяч вiсiмсот дев'яносто")
        # self.assertEqual(
        #     num2words(215461407892039002157189883901676, lang='uk'),
        #     "двiстi п'ятнадцять нонiльйонiв чотириста шiстдесят один "
        #     "октильйон чотириста сiм септильйонiв вiсiмсот дев'яносто "
        #     "два секстильйони тридцять дев'ять квiнтильйонiв два "
        #     "квадрильйони сто п'ятдесят сiм трильйонiв сто вiсiмдесят "
        #     "дев'ять мiльярдiв вiсiмсот вiсiмдесят три мiльйона "
        #     "дев'ятсот одна тисяча шiстсот "
        #     "сiмдесят шiсть")
        # self.assertEqual(
        #     num2words(719094234693663034822824384220291, lang='uk'),
        #     "сiмсот дев'ятнадцять нонiльйонiв дев'яносто чотири октильйони "
        #     "двiстi тридцять чотири септильйони шiстсот дев'яносто три "
        #     "секстильйони шiстсот шiстдесят три квiнтильйони тридцять "
        #     "чотири квадрильйони вiсiмсот двадцять два трильйони вiсiмсот "
        #     "двадцять чотири мiльярди триста вiсiмдесят чотири мiльйона "
        #     "двiстi двадцять тисяч двiстi дев'яносто один")

    def test_and_join_199(self):
        self.assertEqual(num2words(187, lang='uk'), "сто вiсiмдесят сiм")

    def test_cardinal_for_float_number(self):
        self.assertEqual(
            num2words(12.40, lang='uk'), "дванадцять кома чотири"
        )
        self.assertEqual(
            num2words(17.31, lang='uk'), "сiмнадцять кома тридцять одна"
        )
        self.assertEqual(
            num2words(14.13, lang='uk'), "чотирнадцять кома тринадцять"
        )
        self.assertEqual(
            num2words(12.31, lang='uk'), "дванадцять кома тридцять одна"
        )

    def test_to_ordinal(self):
        # @TODO: implement to_ordinal
        with self.assertRaises(NotImplementedError):
            num2words(1, lang='uk', to='ordinal')

    def test_to_currency(self):
        # self.assertEqual(
        #     num2words(1.0, lang='uk', to='currency', currency='EUR'),
        #     "один євро, нуль центiв"
        # )
        self.assertEqual(
            num2words(1.0, lang='uk', to='currency', currency='UAH'),
            "одна гривня, нуль копiйок"
        )
        self.assertEqual(
            num2words(1234.56, lang='uk', to='currency', currency='EUR'),
            "одна тисяча двiстi тридцять чотири євро, п'ятдесят шiсть центiв"
        )
        self.assertEqual(
            num2words(1234.56, lang='uk', to='currency', currency='UAH'),
            "одна тисяча двiстi тридцять чотири гривнi, п'ятдесят шiсть "
            "копiйок"
        )
        # self.assertEqual(
        #     num2words(10111, lang='uk', to='currency', currency='EUR',
        #               separator=u' та'),
        #     "сто один євро та одинадцять центiв"
        # )
        self.assertEqual(
            num2words(10121, lang='uk', to='currency', currency='UAH',
                      separator=u' та'),
            "сто одна гривня та двадцять одна копiйка"
        )
        self.assertEqual(
            num2words(10121, lang='uk', to='currency', currency='UAH',
                      separator=u' та'),
            "сто одна гривня та двадцять одна копiйка"
        )
        self.assertEqual(
            num2words(10122, lang='uk', to='currency', currency='UAH',
                      separator=u' та'),
            "сто одна гривня та двадцять двi копiйки"
        )
        # self.assertEqual(
        #     num2words(10121, lang='uk', to='currency', currency='EUR',
        #               separator=u' та'),
        #     "сто один євро та двадцять один цент"
        # )
        self.assertEqual(
            num2words(-1251985, lang='uk', to='currency', currency='EUR',
                      cents=False),
            "мiнус дванадцять тисяч п'ятсот дев'ятнадцять євро, 85 центiв"
        )
        self.assertEqual(
            num2words('38.4', lang='uk', to='currency', separator=' и',
                      cents=False, currency='EUR'),
            "тридцять вiсiм євро и 40 центiв"
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import division, print_function, unicode_literals

from . import lang_EU


class Num2Word_NO(lang_EU.Num2Word_EU):
    GIGA_SUFFIX = "illard"
    MEGA_SUFFIX = "illion"

    def set_high_numwords(self, high):
        cap = 3 + 6 * len(high)

        for word, n in zip(high, range(cap, 3, -6)):
            if self.GIGA_SUFFIX:
                self.cards[10 ** n] = word + self.GIGA_SUFFIX

            if self.MEGA_SUFFIX:
                self.cards[10 ** (n - 3)] = word + self.MEGA_SUFFIX

    def setup(self):
        super(Num2Word_NO, self).setup()

        self.negword = "minus "
        self.pointword = "komma"
        self.exclude_title = ["og", "komma", "minus"]

        self.mid_numwords = [(1000, "tusen"), (100, "hundre"),
                             (90, "nitti"), (80, "\xe5tti"), (70, "sytti"),
                             (60, "seksti"), (50, "femti"), (40, "f\xf8rti"),
                             (30, "tretti")]
        self.low_numwords = ["tjue", "nitten", "atten", "sytten",
                             "seksten", "femten", "fjorten", "tretten",
                             "tolv", "elleve", "ti", "ni", "\xe5tte",
                             "syv", "seks", "fem", "fire", "tre", "to",
                             "en", "null"]
        self.ords = {"en": "f\xf8rste",
                     "to": "andre",
                     "tre": "tredje",
                     "fire": "fjerde",
                     "fem": "femte",
                     "seks": "sjette",
                     "syv": "syvende",
                     "\xe5tte": "\xe5ttende",
                     "ni": "niende",
                     "ti": "tiende",
                     "elleve": "ellevte",
                     "tolv": "tolvte",
                     "tjue": "tjuende"}

    def merge(self, lpair, rpair):
        ltext, lnum = lpair
        rtext, rnum = rpair
        if lnum == 1 and rnum < 100:
            return (rtext, rnum)
        elif 100 > lnum > rnum:
            return ("%s-%s" % (ltext, rtext), lnum + rnum)
        elif lnum >= 100 > rnum:
            return ("%s og %s" % (ltext, rtext), lnum + rnum)
        elif rnum > lnum:
            return ("%s %s" % (ltext, rtext), lnum * rnum)
        return ("%s, %s" % (ltext, rtext), lnum + rnum)

    def to_ordinal(self, value):
        self.verify_ordinal(value)
        outwords = self.to_cardinal(value).split(" ")
        lastwords = outwords[-1].split("-")
        lastword = lastwords[-1].lower()
        try:
            lastword = self.ords[lastword]
        except KeyError:
            if lastword[-2:] == "ti":
                lastword = lastword + "ende"
            else:
                lastword += "de"
        lastwords[-1] = self.title(lastword)
        outwords[-1] = "".join(lastwords)
        return " ".join(outwords)

    def to_ordinal_num(self, value):
        self.verify_ordinal(value)
        return "%s%s" % (value, self.to_ordinal(value)[-2:])

    def to_year(self, val, longval=True):
        if not (val // 100) % 10:
            return self.to_cardinal(val)
        return self.to_splitnum(val, hightxt="hundre", jointxt="og",
                                longval=longval)

    def to_currency(self, val, longval=True):
        return self.to_splitnum(val, hightxt="krone/r", lowtxt="\xf8re/r",
                                jointxt="og", longval=longval, cents=True)



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from unittest import TestCase

from num2words import num2words


class Num2WordsARTest(TestCase):

    def test_default_currency(self):
        self.assertEqual(num2words(1, to='currency', lang='ar'), 'واحد ريال')
        self.assertEqual(num2words(2, to='currency', lang='ar'),
                         'اثنان ريالان')
        self.assertEqual(num2words(10, to='currency', lang='ar'),
                         'عشرة ريالات')
        self.assertEqual(num2words(100, to='currency', lang='ar'), 'مائة ريال')
        self.assertEqual(num2words(652.12, to='currency', lang='ar'),
                         'ستمائة و اثنان و خمسون ريالاً و اثنتا عشرة هللة')
        self.assertEqual(num2words(324, to='currency', lang='ar'),
                         'ثلاثمائة و أربعة و عشرون ريالاً')
        self.assertEqual(num2words(2000, to='currency', lang='ar'),
                         'ألفا ريال')
        self.assertEqual(num2words(541, to='currency', lang='ar'),
                         'خمسمائة و واحد و أربعون ريالاً')
        self.assertEqual(num2words(10000, to='currency', lang='ar'),
                         'عشرة آلاف ريال')
        self.assertEqual(num2words(20000.12, to='currency', lang='ar'),
                         'عشرون ألف ريال و اثنتا عشرة هللة')
        self.assertEqual(num2words(1000000, to='currency', lang='ar'),
                         'واحد مليون ريال')
        val = 'تسعمائة و ثلاثة و عشرون ألفاً  و أربعمائة و أحد عشر ريالاً'
        self.assertEqual(num2words(923411, to='currency', lang='ar'), val)
        self.assertEqual(num2words(63411, to='currency', lang='ar'),
                         'ثلاثة و ستون ألفاً  و أربعمائة و أحد عشر ريالاً')
        self.assertEqual(num2words(1000000.99, to='currency', lang='ar'),
                         'واحد مليون ريال و تسع و تسعون هللة')

    def test_currency_parm(self):
        self.assertEqual(
            num2words(1, to='currency', lang='ar', currency="KWD"),
            'واحد دينار')
        self.assertEqual(
            num2words(10, to='currency', lang='ar', currency="EGP"),
            'عشرة جنيهات')
        self.assertEqual(
            num2words(20000.12, to='currency', lang='ar', currency="EGP"),
            'عشرون ألف جنيه و اثنتا عشرة قرش')
        self.assertEqual(
            num2words(923411, to='currency', lang='ar', currency="SR"),
            'تسعمائة و ثلاثة و عشرون ألفاً  و أربعمائة و أحد عشر ريالاً')
        self.assertEqual(
            num2words(1000000.99, to='currency', lang='ar', currency="KWD"),
            'واحد مليون دينار و تسع و تسعون فلس')

    def test_ordinal(self):
        self.assertEqual(num2words(1, to='ordinal', lang='ar'), 'اول')
        self.assertEqual(num2words(2, to='ordinal', lang='ar'), 'ثاني')
        self.assertEqual(num2words(3, to='ordinal', lang='ar'), 'ثالث')
        self.assertEqual(num2words(4, to='ordinal', lang='ar'), 'رابع')
        self.assertEqual(num2words(5, to='ordinal', lang='ar'), 'خامس')
        self.assertEqual(num2words(6, to='ordinal', lang='ar'), 'سادس')
        self.assertEqual(num2words(9, to='ordinal', lang='ar'), 'تاسع')
        self.assertEqual(num2words(20, to='ordinal', lang='ar'), 'عشرون')
        self.assertEqual(num2words(94, to='ordinal', lang='ar'),
                         'أربع و تسعون')
        self.assertEqual(num2words(102, to='ordinal', lang='ar'),
                         'مائة و اثنان')
        self.assertEqual(
            num2words(923411, to='ordinal_num', lang='ar'),
            'تسعمائة و ثلاثة و عشرون ألفاً  و أربعمائة و أحد عشر')

    def test_cardinal(self):
        self.assertEqual(num2words(12, to='cardinal', lang='ar'), 'اثنا عشر')
        self.assertEqual(num2words(-8324, to='cardinal', lang='ar'),
                         'سالب ثمانية آلاف  و ثلاثمائة و أربعة و عشرون')
        self.assertEqual(
            num2words(3431.12, to='cardinal', lang='ar'),
            'ثلاثة آلاف  و أربعمائة و واحد و ثلاثون  , اثنتا عشرة')
        self.assertEqual(num2words(431, to='cardinal', lang='ar'),
                         'أربعمائة و واحد و ثلاثون')
        self.assertEqual(num2words(94231, to='cardinal', lang='ar'),
                         'أربعة و تسعون ألفاً  و مئتان و واحد و ثلاثون')
        self.assertEqual(num2words(1431, to='cardinal', lang='ar'),
                         'واحد ألف  و أربعمائة و واحد و ثلاثون')

    def test_prefix_and_suffix(self):
        self.assertEqual(num2words(645, to='currency',
                                   lang='ar', prefix="فقط", suffix="لاغير"),
                         'فقط ستمائة و خمسة و أربعون ريالاً لاغير')

    def test_year(self):
        self.assertEqual(num2words(2000, to='year', lang='ar'), 'ألفا')

    def test_max_numbers(self):
        with self.assertRaises(Exception) as context:
            num2words(10 ** 36, to='year', lang='ar')

        self.assertTrue('Too large' in str(context.exception))



# -*- coding: utf-8 -*-
# Copyright (c) 2017, Tufan Kaynak, Framras.  All Rights Reserved.
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from unittest import TestCase

from num2words import num2words


class Num2WordsTRTest(TestCase):
    def test_tr(self):
        # ref https://github.com/savoirfairelinux/num2words/issues/8

        self.assertEqual(num2words(1, True, "tr"), u"birinci")
        self.assertEqual(num2words(2, True, "tr"), u"ikinci")
        self.assertEqual(num2words(9, True, "tr"), u"dokuzuncu")
        self.assertEqual(num2words(10, True, "tr"), u"onuncu")
        self.assertEqual(num2words(11, True, "tr"), u"onbirinci")
        self.assertEqual(num2words(44, True, "tr"), u"kırkdördüncü")
        self.assertEqual(num2words(100, True, "tr"), u"yüzüncü")
        self.assertEqual(num2words(101, True, "tr"), u"yüzbirinci")
        self.assertEqual(num2words(103, True, "tr"), u"yüzüçüncü")
        self.assertEqual(num2words(110, True, "tr"), u"yüzonuncu")
        self.assertEqual(num2words(111, True, "tr"), u"yüzonbirinci")
        self.assertEqual(num2words(1000, True, "tr"), u"bininci")
        self.assertEqual(num2words(1001, True, "tr"), u"binbirinci")
        self.assertEqual(num2words(1010, True, "tr"), u"binonuncu")
        self.assertEqual(num2words(1011, True, "tr"), u"binonbirinci")
        self.assertEqual(num2words(1100, True, "tr"), u"binyüzüncü")
        self.assertEqual(num2words(1110, True, "tr"), u"binyüzonuncu")
        self.assertEqual(
            num2words(2341, True, "tr"), u"ikibinüçyüzkırkbirinci"
        )
        self.assertEqual(num2words(10000, True, "tr"), u"onbininci")
        self.assertEqual(num2words(10010, True, "tr"), u"onbinonuncu")
        self.assertEqual(num2words(10100, True, "tr"), u"onbinyüzüncü")
        self.assertEqual(num2words(10110, True, "tr"), u"onbinyüzonuncu")
        self.assertEqual(num2words(11000, True, "tr"), u"onbirbininci")
        self.assertEqual(num2words(35000, True, "tr"), u"otuzbeşbininci")
        self.assertEqual(
            num2words(116331, True, "tr"), u"yüzonaltıbinüçyüzotuzbirinci"
        )
        self.assertEqual(
            num2words(116330, True, "tr"), u"yüzonaltıbinüçyüzotuzuncu"
        )
        self.assertEqual(num2words(100000, True, "tr"), u"yüzbininci")
        self.assertEqual(num2words(501000, True, "tr"), u"beşyüzbirbininci")
        self.assertEqual(
            num2words(1000111, True, "tr"), u"birmilyonyüzonbirinci"
        )
        self.assertEqual(
            num2words(111000111, True, "tr"), u"yüzonbirmilyonyüzonbirinci"
        )
        self.assertEqual(
            num2words(111001111, True, "tr"), u"yüzonbirmilyonbinyüzonbirinci"
        )
        self.assertEqual(
            num2words(111111111, True, "tr"),
            u"yüzonbirmilyonyüzonbirbinyüzonbirinci"
        )
        self.assertEqual(num2words(100001000, True, "tr"), u"yüzmilyonbininci")
        self.assertEqual(
            num2words(100001001, True, "tr"), u"yüzmilyonbinbirinci"
        )
        self.assertEqual(
            num2words(100010000, True, "tr"), u"yüzmilyononbininci"
        )
        self.assertEqual(
            num2words(100010001, True, "tr"), u"yüzmilyononbinbirinci"
        )
        self.assertEqual(
            num2words(100011000, True, "tr"), u"yüzmilyononbirbininci"
        )
        self.assertEqual(
            num2words(100011001, True, "tr"), u"yüzmilyononbirbinbirinci"
        )
        self.assertEqual(
            num2words(101011001, True, "tr"), u"yüzbirmilyononbirbinbirinci"
        )
        self.assertEqual(
            num2words(101011010, True, "tr"), u"yüzbirmilyononbirbinonuncu"
        )
        self.assertEqual(
            num2words(1101011010, True, "tr"),
            u"birmilyaryüzbirmilyononbirbinonuncu"
        )
        self.assertEqual(
            num2words(101101011010, True, "tr"),
            u"yüzbirmilyaryüzbirmilyononbirbinonuncu"
        )
        self.assertEqual(
            num2words(1000000000001, True, "tr"), u"birtrilyonbirinci"
        )

        self.assertEqual(num2words(1, False, "tr"), u"bir")
        self.assertEqual(num2words(2, False, "tr"), u"iki")
        self.assertEqual(num2words(9, False, "tr"), u"dokuz")
        self.assertEqual(num2words(10, False, "tr"), u"on")
        self.assertEqual(num2words(11, False, "tr"), u"onbir")
        self.assertEqual(num2words(44, False, "tr"), u"kırkdört")
        self.assertEqual(num2words(100, False, "tr"), u"yüz")
        self.assertEqual(num2words(101, False, "tr"), u"yüzbir")
        self.assertEqual(num2words(103, False, "tr"), u"yüzüç")
        self.assertEqual(num2words(110, False, "tr"), u"yüzon")
        self.assertEqual(num2words(111, False, "tr"), u"yüzonbir")
        self.assertEqual(num2words(1000, False, "tr"), u"bin")
        self.assertEqual(num2words(1001, False, "tr"), u"binbir")
        self.assertEqual(num2words(1010, False, "tr"), u"binon")
        self.assertEqual(num2words(1011, False, "tr"), u"binonbir")
        self.assertEqual(num2words(1100, False, "tr"), u"binyüz")
        self.assertEqual(num2words(1110, False, "tr"), u"binyüzon")
        self.assertEqual(num2words(2341, False, "tr"), u"ikibinüçyüzkırkbir")
        self.assertEqual(num2words(10000, False, "tr"), u"onbin")
        self.assertEqual(num2words(10010, False, "tr"), u"onbinon")
        self.assertEqual(num2words(10100, False, "tr"), u"onbinyüz")
        self.assertEqual(num2words(10110, False, "tr"), u"onbinyüzon")
        self.assertEqual(num2words(11000, False, "tr"), u"onbirbin")
        self.assertEqual(num2words(35000, False, "tr"), u"otuzbeşbin")
        self.assertEqual(
            num2words(116331, False, "tr"), u"yüzonaltıbinüçyüzotuzbir"
        )
        self.assertEqual(
            num2words(116330, False, "tr"), u"yüzonaltıbinüçyüzotuz"
        )
        self.assertEqual(num2words(500000, False, "tr"), u"beşyüzbin")
        self.assertEqual(num2words(501000, False, "tr"), u"beşyüzbirbin")
        self.assertEqual(num2words(1000111, False, "tr"), u"birmilyonyüzonbir")
        self.assertEqual(
            num2words(111000111, False, "tr"), u"yüzonbirmilyonyüzonbir"
        )
        self.assertEqual(
            num2words(111001111, False, "tr"), u"yüzonbirmilyonbinyüzonbir"
        )
        self.assertEqual(
            num2words(111111111, False, "tr"),
            u"yüzonbirmilyonyüzonbirbinyüzonbir"
        )
        self.assertEqual(num2words(100001000, False, "tr"), u"yüzmilyonbin")
        self.assertEqual(num2words(100001001, False, "tr"), u"yüzmilyonbinbir")
        self.assertEqual(num2words(100010000, False, "tr"), u"yüzmilyononbin")
        self.assertEqual(
            num2words(100010001, False, "tr"), u"yüzmilyononbinbir"
        )
        self.assertEqual(
            num2words(100011000, False, "tr"), u"yüzmilyononbirbin"
        )
        self.assertEqual(
            num2words(100011001, False, "tr"), u"yüzmilyononbirbinbir"
        )
        self.assertEqual(
            num2words(101011001, False, "tr"), u"yüzbirmilyononbirbinbir"
        )
        self.assertEqual(
            num2words(101011010, False, "tr"), u"yüzbirmilyononbirbinon"
        )
        self.assertEqual(
            num2words(1101011010, False, "tr"),
            u"birmilyaryüzbirmilyononbirbinon"
        )
        self.assertEqual(
            num2words(101101011010, False, "tr"),
            u"yüzbirmilyaryüzbirmilyononbirbinon"
        )
        self.assertEqual(
            num2words(1000000000001, False, "tr"), u"birtrilyonbir"
        )
        self.assertEqual(num2words(0.01, False, "tr"), u"sıfırvirgülbir")
        self.assertEqual(num2words(0.1, False, "tr"), u"sıfırvirgülon")
        self.assertEqual(num2words(0.21, False, "tr"), u"sıfırvirgülyirmibir")
        self.assertEqual(num2words(1.01, False, "tr"), u"birvirgülbir")
        self.assertEqual(num2words(1.1, False, "tr"), u"birvirgülon")
        self.assertEqual(num2words(1.21, False, "tr"), u"birvirgülyirmibir")
        self.assertEqual(
            num2words(101101011010.02, False, "tr"),
            u"yüzbirmilyaryüzbirmilyononbirbinonvirgüliki"
        )
        self.assertEqual(
            num2words(101101011010.2, False, "tr"),
            u"yüzbirmilyaryüzbirmilyononbirbinonvirgülyirmi"
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2003, Taro Ogawa.  All Rights Reserved.
# Copyright (c) 2013, Savoir-faire Linux inc.  All Rights Reserved.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301 USA

from __future__ import unicode_literals

from .base import Num2Word_Base
from .utils import get_digits, splitbyx

ZERO = ('нуль',)

ONES_FEMININE = {
    1: ('одна',),
    2: ('двi',),
    3: ('три',),
    4: ('чотири',),
    5: ('п\'ять',),
    6: ('шiсть',),
    7: ('сiм',),
    8: ('вiсiм',),
    9: ('дев\'ять',),
}

ONES = {
    1: ('один',),
    2: ('два',),
    3: ('три',),
    4: ('чотири',),
    5: ('п\'ять',),
    6: ('шiсть',),
    7: ('сiм',),
    8: ('вiсiм',),
    9: ('дев\'ять',),
}

TENS = {
    0: ('десять',),
    1: ('одинадцять',),
    2: ('дванадцять',),
    3: ('тринадцять',),
    4: ('чотирнадцять',),
    5: ('п\'ятнадцять',),
    6: ('шiстнадцять',),
    7: ('сiмнадцять',),
    8: ('вiсiмнадцять',),
    9: ('дев\'ятнадцять',),
}

TWENTIES = {
    2: ('двадцять',),
    3: ('тридцять',),
    4: ('сорок',),
    5: ('п\'ятдесят',),
    6: ('шiстдесят',),
    7: ('сiмдесят',),
    8: ('вiсiмдесят',),
    9: ('дев\'яносто',),
}

HUNDREDS = {
    1: ('сто',),
    2: ('двiстi',),
    3: ('триста',),
    4: ('чотириста',),
    5: ('п\'ятсот',),
    6: ('шiстсот',),
    7: ('сiмсот',),
    8: ('вiсiмсот',),
    9: ('дев\'ятсот',),
}

THOUSANDS = {
    1: ('тисяча', 'тисячi', 'тисяч'),  # 10^3
    2: ('мiльйон', 'мiльйони', 'мiльйонiв'),  # 10^6
    3: ('мiльярд', 'мiльярди', 'мiльярдiв'),  # 10^9
    4: ('трильйон', 'трильйони', 'трильйонiв'),  # 10^12
    5: ('квадрильйон', 'квадрильйони', 'квадрильйонiв'),  # 10^15
    6: ('квiнтильйон', 'квiнтильйони', 'квiнтильйонiв'),  # 10^18
    7: ('секстильйон', 'секстильйони', 'секстильйонiв'),  # 10^21
    8: ('септильйон', 'септильйони', 'септильйонiв'),  # 10^24
    9: ('октильйон', 'октильйони', 'октильйонiв'),  # 10^27
    10: ('нонiльйон', 'нонiльйони', 'нонiльйонiв'),  # 10^30
}


class Num2Word_UK(Num2Word_Base):
    CURRENCY_FORMS = {
        'UAH': (
            ('гривня', 'гривнi', 'гривень'),
            ('копiйка', 'копiйки', 'копiйок')
        ),
        'EUR': (
            ('євро', 'євро', 'євро'), ('цент', 'центи', 'центiв')
        ),
    }

    def setup(self):
        self.negword = "мiнус"
        self.pointword = "кома"

    def to_cardinal(self, number):
        n = str(number).replace(',', '.')
        if '.' in n:
            left, right = n.split('.')
            return '%s %s %s' % (
                self._int2word(int(left)),
                self.pointword,
                self._int2word(int(right))
            )
        else:
            return self._int2word(int(n))

    def pluralize(self, n, forms):
        if n % 100 < 10 or n % 100 > 20:
            if n % 10 == 1:
                form = 0
            elif 5 > n % 10 > 1:
                form = 1
            else:
                form = 2
        else:
            form = 2

        return forms[form]

    def _int2word(self, n, feminine=True):
        if n < 0:
            return ' '.join([self.negword, self._int2word(abs(n))])

        if n == 0:
            return ZERO[0]

        words = []
        chunks = list(splitbyx(str(n), 3))
        i = len(chunks)
        for x in chunks:
            i -= 1

            if x == 0:
                continue

            n1, n2, n3 = get_digits(x)

            if n3 > 0:
                words.append(HUNDREDS[n3][0])

            if n2 > 1:
                words.append(TWENTIES[n2][0])

            if n2 == 1:
                words.append(TENS[n1][0])
            # elif n1 > 0 and not (i > 0 and x == 1):
            elif n1 > 0:
                ones = ONES_FEMININE if i == 1 or feminine and i == 0 else ONES
                words.append(ones[n1][0])

            if i > 0:
                words.append(self.pluralize(x, THOUSANDS[i]))

        return ' '.join(words)

    def _cents_verbose(self, number, currency):
        return self._int2word(number, currency == 'UAH')

    def to_ordinal(self, number):
        raise NotImplementedError()
